==> eccodes_2.41.0.orig.tar.xz: eccodes-2.41.0/src/eccodes/accessor/BudgDate.cc <==
/*
 * (C) Copyright 2005- ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
 * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 */

#include "BudgDate.h"

eccodes::accessor::BudgDate _grib_accessor_budgdate;
eccodes::Accessor* grib_accessor_budgdate = &_grib_accessor_budgdate;

namespace eccodes::accessor
{

void BudgDate::init(const long l, grib_arguments* c)
{
    Long::init(l, c);
    int n = 0;

    year_  = c->get_name(get_enclosing_handle(), n++);
    month_ = c->get_name(get_enclosing_handle(), n++);
    day_   = c->get_name(get_enclosing_handle(), n++);
}

int BudgDate::unpack_long(long* val, size_t* len)
{
    int ret = 0;

    long year  = 0;
    long month = 0;
    long day   = 0;

    if ((ret = grib_get_long_internal(get_enclosing_handle(), day_, &day)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_get_long_internal(get_enclosing_handle(), month_, &month)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_get_long_internal(get_enclosing_handle(), year_, &year)) != GRIB_SUCCESS)
        return ret;

    if (*len < 1)
        return GRIB_WRONG_ARRAY_SIZE;

    val[0] = (1900 + year) * 10000 + month * 100 + day;

    return ret;
}

/* TODO: Check for a valid date */
int BudgDate::pack_long(const long* val, size_t* len)
{
    int ret = 0;
    long v  = val[0];

    long year  = 0;
    long month = 0;
    long day   = 0;

    if (*len != 1)
        return GRIB_WRONG_ARRAY_SIZE;

    year = v / 10000;
    v %= 10000;
    month = v / 100;
    v %= 100;
    day = v;

    year -= 1900;

    ECCODES_ASSERT(year < 255);

    if ((ret = grib_set_long_internal(get_enclosing_handle(), day_, day)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_set_long_internal(get_enclosing_handle(), month_, month)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_set_long_internal(get_enclosing_handle(), year_, year)) != GRIB_SUCCESS)
        return ret;

    return ret;
}

}  // namespace eccodes::accessor

==> eccodes_2.44.1.orig.tar.xz: eccodes-2.44.1/src/eccodes/accessor/BudgDate.cc <==
/*
 * (C) Copyright 2005- ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
 * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 */

#include "BudgDate.h"

eccodes::accessor::BudgDate _grib_accessor_budgdate;
eccodes::Accessor* grib_accessor_budgdate = &_grib_accessor_budgdate;

namespace eccodes::accessor
{

void BudgDate::init(const long l, grib_arguments* c)
{
    Long::init(l, c);
    int n = 0;

    year_  = c->get_name(get_enclosing_handle(), n++);
    month_ = c->get_name(get_enclosing_handle(), n++);
    day_   = c->get_name(get_enclosing_handle(), n++);
}

int BudgDate::unpack_long(long* val, size_t* len)
{
    int ret = 0;

    long year  = 0;
    long month = 0;
    long day   = 0;

    if ((ret = grib_get_long_internal(get_enclosing_handle(), day_, &day)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_get_long_internal(get_enclosing_handle(), month_, &month)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_get_long_internal(get_enclosing_handle(), year_, &year)) != GRIB_SUCCESS)
        return ret;

    if (*len < 1)
        return GRIB_WRONG_ARRAY_SIZE;

    val[0] = (1900 + year) * 10000 + month * 100 + day;

    return ret;
}

/* TODO(masn): Check for a valid date */
int BudgDate::pack_long(const long* val, size_t* len)
{
    int ret = 0;
    long v  = val[0];

    long year  = 0;
    long month = 0;
    long day   = 0;

    if (*len != 1)
        return GRIB_WRONG_ARRAY_SIZE;

    year = v / 10000;
    v %= 10000;
    month = v / 100;
    v %= 100;
    day = v;

    year -= 1900;

    ECCODES_ASSERT(year < 255);

    if ((ret = grib_set_long_internal(get_enclosing_handle(), day_, day)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_set_long_internal(get_enclosing_handle(), month_, month)) != GRIB_SUCCESS)
        return ret;
    if ((ret = grib_set_long_internal(get_enclosing_handle(), year_, year)) != GRIB_SUCCESS)
        return ret;

    return ret;
}

}  // namespace eccodes::accessor
